#pragma once

#include <vector>
#include "J3ML/LinearAlgebra/Vector3.h"
#include <glad/glad.h>

namespace LearnOpenGL
{
    // Renders a reference floor-plane grid in 3D space
    class Grid
    {
    public:
        Grid() {
            setup_buffers();
        }
        ~Grid() {
            //glDeleteVertexArrays(1, &VAO);
            glDeleteBuffers(1, &VBO);
        }
        void Render() const
        {
            //glBindVertexArray(VAO);
            glDrawArrays(GL_LINES, 0, (int)vertex_count);
            //glBindVertexArray(0);
        }
    protected:
    private:
        unsigned int VAO{}, VBO{};
        unsigned int vertex_count{};
        // Adapted from OpenGL Shading Language Cookbook by David Wolff
        void setup_buffers()
        {
            std::vector<J3ML::LinearAlgebra::Vector3> vertices;
            float size = 10.f;
            const int GRID_DIMS = 10;

            for (auto column = 0; column <= GRID_DIMS * 2; column++)
            {
                float x = (float)column - size;
                vertices.emplace_back(x, 0, -size);
                vertices.emplace_back(x, 0, size);
            }
            for (auto row = 0; row <= GRID_DIMS * 2; row++)
            {
                float z = (float)row - size;
                vertices.emplace_back(-size, 0, z);
                vertices.emplace_back(size, 0, z);
            }

            vertex_count = vertices.size();

            //glGenVertexArrays(1, &VAO);
            glGenBuffers(1, &VBO);

            //glBindVertexArray(VAO);
            glBindBuffer(GL_ARRAY_BUFFER, VBO);
            glBufferData(GL_ARRAY_BUFFER, vertices.size()* sizeof(J3ML::LinearAlgebra::Vector3), vertices.data(), GL_STATIC_DRAW);


            glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, nullptr);
            glEnableVertexAttribArray(0);
            //glBindVertexArray(0);
        }
    };
}