#include <LearnOpenGL/Animation.h>
namespace LearnOpenGL
{

    Animation::Animation(const std::string &animationPath, Model *model) {
        Assimp::Importer importer;

        const aiScene* scene = importer.ReadFile(animationPath, aiProcess_Triangulate);
        assert(scene && scene->mRootNode);
        auto animation = scene->mAnimations[0];
        Duration = animation->mDuration;
        TicksPerSecond = animation->mTicksPerSecond;
        aiMatrix4x4 globalTransformation = scene->mRootNode->mTransformation;
        globalTransformation = globalTransformation.Inverse();
        ReadHierarchyData(RootNode, scene->mRootNode);
        ReadMissingBones(animation, *model);
    }

    Animation::~Animation() { }

    Bone *Animation::FindBone(const std::string &name) {
        auto iter = std::find_if(Bones.begin(), Bones.end(),
                                 [&](const Bone& bone)
                                 {
                                     return bone.GetBoneName() == name;
                                 }
        );
        if (iter == Bones.end()) return nullptr;
        else return &(*iter);
    }

    void Animation::ReadMissingBones(const aiAnimation *animation, Model &model) {
        int size = animation->mNumChannels;

        auto boneInfoMap = model.GetBoneInfoMap();
        int boneCount = model.GetBoneCount();

        // Reading channels (bones engaged in an animation and their keyframes)
        for (int i = 0; i < size; i++)
        {
            auto channel = animation->mChannels[i];
            std::string boneName = channel->mNodeName.data;

            if (boneInfoMap.find(boneName) == boneInfoMap.end())
            {
                boneInfoMap[boneName].ID = boneCount;
                boneCount++;
            }
            Bones.push_back(Bone(channel->mNodeName.data, boneInfoMap[channel->mNodeName.data].ID, channel));
        }
        BoneInfoMap = boneInfoMap;
    }

    void Animation::ReadHierarchyData(AssimpNodeData &dest, const aiNode *src) {
        assert(src);

        dest.Name = src->mName.data;
        dest.Transformation = Matrix4x4(&(src->mTransformation.a1));
        dest.ChildCount = src->mNumChildren;

        for (int i = 0; i < src->mNumChildren; i++)
        {
            AssimpNodeData newData;
            ReadHierarchyData(newData, src->mChildren[i]);
            dest.Children.push_back(newData);
        }
    }
}