#include <LearnOpenGL/Animator.h>

namespace LearnOpenGL
{

    Animator::Animator(Animation *animation) {
        this->CurrentTime = 0.0;
        this->CurrentAnimation = animation;

        this->FinalBoneMatrices.reserve(100);

        for (int i = 0; i < 100; i++)
            this->FinalBoneMatrices.push_back(Matrix4x4(1.f));
    }

    void Animator::UpdateAnimations(float dt) {
        this->DeltaTime = dt;
        if (this->CurrentAnimation != nullptr)
        {
            this->CurrentTime += this->CurrentAnimation->GetTicksPerSecond() * dt;
            this->CurrentTime = fmod(this->CurrentTime, this->CurrentAnimation->GetDuration());
            CalculateBoneTransform(&this->CurrentAnimation->GetRootNode(), Matrix4x4(1.f));
        }
    }

    void Animator::PlayAnimation(Animation *pAnimation) {
        this->CurrentAnimation = pAnimation;
        this->CurrentTime = 0.0f;
    }

    void Animator::CalculateBoneTransform(const AssimpNodeData *node, Matrix4x4 parentTransform) {
        std::string nodeName = node->Name;
        Matrix4x4 nodeTransform = node->Transformation;


        Bone* bone = CurrentAnimation->FindBone(nodeName);
        if (bone != nullptr)
        {
            bone->Update(this->CurrentTime);
            nodeTransform = bone->GetLocalTransform();
        }

        Matrix4x4 globalTransformation = parentTransform * nodeTransform;

        auto boneInfoMap = this->CurrentAnimation->GetBoneIDMap();
        if (boneInfoMap.find(nodeName) != boneInfoMap.end())
        {
            int index = boneInfoMap[nodeName].ID;
            Matrix4x4 offset = boneInfoMap[nodeName].Offset;
            this->FinalBoneMatrices[index] = globalTransformation * offset;
        }

        for (int i = 0; i < node->ChildCount; i++)
        {
            CalculateBoneTransform(&node->Children[i], globalTransformation);
        }
    }

    std::vector<Matrix4x4> Animator::GetFinalBoneMatrices() {
        return this->FinalBoneMatrices;
    }
}